package chat.server;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

// This is a simple chat application. It allows clients to enroll
// under a particular name, and send messages to each other.
// Messages are sent to the client via the ChatClient interface.

public class ChatServer
{
// clients is a table that maps a client name to a ChatClient 
// interface
     protected Hashtable clients;

     public ChatServer()
     {
          clients = new Hashtable();
     }

// Add client adds a new client to the system and tells the other
// clients about the new client.

     public synchronized void addClient(String name, 
ChatClient client)
     {

// If the client picks a name that is already here, 
// disconnect the new client, let the old one keep its name.

          if (clients.get(name) != null) {
               client.disconnect();
               return;
          }

// Add the new client to the table
          clients.put(name, client);

// Tell the other clients about this new client
          sendEnterMessage(name);
     }
     
     public synchronized void removeClient(String name)
     {
          ChatClient client = (ChatClient) clients.get(name);
          if (client != null) {
               clients.remove(name);
               sendLeaveMessage(name);
          }
     }

// removeClient removes a client from the chat system and tells
// the other clients about it.

     public synchronized void removeClient(ChatClient client)
     {

// We remove by ChatClient, not by name. We have to enumerate through
// all the clients to find out the name of this client.

          Enumeration e = clients.keys();

          while (e.hasMoreElements()) {
               String key = (String) e.nextElement();

// If we found the right name for this client, remove them and
// tell everyone about it.
               if (clients.get(key) == client) {
                    clients.remove(key);
                    sendLeaveMessage(key);
               }
          }
     }

// sendChat is called by a client to send a message to the 
// other clients

     public synchronized void sendChat(String name, String message)
     {
          Enumeration e = clients.elements();

// Enumerate through all the clients and send them the chat message
// Note that this will send a message back to the original 
// sender, too.

          while (e.hasMoreElements()) {
               ChatClient client = (ChatClient) e.nextElement();

               client.incomingChat(name, message);
          }
     }

// sendEnterMessage tells all the clients when a new client 
// has arrived

     public synchronized void sendEnterMessage(String name)
     {
          Enumeration e = clients.elements();

// Enumerate through all the clients and tell them about 
// the new client

          while (e.hasMoreElements()) {
               ChatClient client = (ChatClient) e.nextElement();

               client.userHasEntered(name);
          }
     }

// sendLeaveMessage tells all the clients that a client has left

     public synchronized void sendLeaveMessage(String name)
     {
          Enumeration e = clients.elements();

// Enumerate through all the clients and tell them who left

          while (e.hasMoreElements()) {
               ChatClient client = (ChatClient) e.nextElement();

               client.userHasLeft(name);
          }
     }

// getUserList returns a list of all the users on the system

     public synchronized String[] getUserList()
     {
          Enumeration e = clients.keys();

// Create an array to hold the user names
          String[] nameList = new String[clients.size()];

// Copy the user names into the nameList array
          int i = 0;
          while (e.hasMoreElements()) {
              nameList[i++] = (String) e.nextElement();
          }

// Return the name list
          return nameList;
     }
}