import java.net.*;
import java.io.*;

import java.rmi.server.UnicastRemoteObject;
import java.rmi.RMISecurityManager;

 chat.server.*;
import chat.rmi.*;

// This class is an RMI client for the chat application

public class RMIChatClientImpl extends UnicastRemoteObject
implements RMIChatClient
{
     public RMIChatClientImpl()
     throws java.rmi.RemoteException
     {
     }

// The following 4 methods are callbacks from the 
// RMIChatServerImpl class.

     public void userHasEntered(String who)
     throws java.rmi.RemoteException
     {
          System.out.println("--- "+who+" has just entered ---");
     }
     
     public void userHasLeft(String who)
     throws java.rmi.RemoteException
     {
          System.out.println("--- "+who+" has just left ---");
     }
     
     public void incomingChat(String who, String chat)
     throws java.rmi.RemoteException
     {
          System.out.println("<"+who+"> "+chat);
     }

     public void disconnect()
     throws java.rmi.RemoteException
     {
          System.out.println("Chat server connection closed.");
          System.exit(0);
     }

     public static void main(String args[])
     {
// Get the name of the enroll factory

          String chatName = System.getProperty("rmiName", "chat");

// Must have a stub security manager!
          System.setSecurityManager(new StubSecurityManager());

          try {

// Get the name the user wants to use
               System.out.print("What name do you want to use? ");

               System.out.flush();

               DataInputStream userInputStream =
                    new DataInputStream(System.in);

               String myName = userInputStream.readLine();

// Create an instance of this object to receive callbacks
               RMIChatClient thisClient = new RMIChatClientImpl();

// Locate the RMIChatEnrol object
               RMIChatEnrol enrol = (RMIChatEnrol)
                    java.rmi.Naming.lookup(chatName);

// Enrol to the chat system
               RMIChatServer server = enrol.enrol(myName, thisClient);

// Free up the enrol object, we don't need it any more
               enrol = null;

// Read lines from the user and pass them to the server

               while (true) {

                    String chatLine = userInputStream.readLine();

                    server.sendChat(chatLine);

               }

          } catch (Exception e) {
               System.out.println("Got exception:");
               e.printStackTrace();
               System.exit(1);
          }
     }
}