package chat.rmi;

import java.rmi.server.UnicastRemoteObject;
import java.rmi.RMISecurityManager;

 chat.server.*;

// This class is actually an RMI encapsulation for the
// ChatClient interface. It implements the methods in the
// ChatClient interface and invokes the corresponding method
// in the RMIChatClient interface.

// It also handles messages coming from the client. When the
// sendChat method is invoked via RMI, it turns around and
// invokes sendChat in the chat application.

public class RMIChatServerImpl extends UnicastRemoteObject
     implements RMIChatServer, ChatClient
{
     protected ChatServer server;
     protected String name;
     protected RMIChatClient client;

     public RMIChatServerImpl(ChatServer server, String name,
          RMIChatClient client)
     throws java.rmi.RemoteException
     {
          this.server = server;
          this.name = name;
          this.client = client;

          server.addClient(name, this);
     }

     public void incomingChat(String who, String chat)
     {
          try {
               client.incomingChat(who, chat);
          } catch (Exception e) {
               try {
                    client.disconnect();
               } catch (Exception ignore) {
               }
               server.removeClient(name);
               client = null;
          }
     }

     public void userHasEntered(String who)
     {
          try {
               client.userHasEntered(who);
          } catch (Exception e) {
               try {
                    client.disconnect();
               } catch (Exception ignore) {
               }
               server.removeClient(name);
               client = null;
          }
     }

     public void userHasLeft(String who)
     {
          try {
               client.userHasLeft(who);
          } catch (Exception e) {
               try {
                    client.disconnect();
               } catch (Exception ignore) {
               }
               server.removeClient(name);
               client = null;
          }
     }

     public void disconnect()
     {
          try {
               client.disconnect();
          } catch (Exception ignore) {
          }
          server.removeClient(name);
          client = null;
     }

     public void sendChat(String chat)
          throws java.rmi.RemoteException
     {
          server.sendChat(name, chat);
     }
}