package chat.tcp.client;

import java.io.*;

 chat.server.*;
import chat.tcp.common.TCPChatMessageTypes;

// This class sets up a thread that reads messages from the
// TCPChatServer and then invokes methods in an object
// implementing the ChatClient interface.

public class TCPChatReader extends Object implements Runnable
{
     protected ChatClient client;
     protected DataInputStream inStream;
     protected Thread myThread;

     public TCPChatReader(ChatClient client, 
DataInputStream inStream)
     {
          this.client = client;
          this.inStream = inStream;
     }

     public void run()
     {
          while (true) {
              try {
                    int messageType = inStream.readInt();

// Look at the message type and call the appropriate method to
// read the message.

                    switch (messageType) {
                         case TCPChatMessageTypes.CHAT:
                             readChat();
                              break;

                         case TCPChatMessageTypes.ENTER:
                              readEnter();
                              break;

                         case TCPChatMessageTypes.LEAVE:
                              readLeave();
                              break;

                         default:
                              skipMessage();
                              break;
                    }
               } catch (Exception e) {
                    client.disconnect();
               }
          }
     }

     public void start()
     {
          myThread = new Thread(this);
          myThread.start();
     }

     public void stop()
     {
          myThread.stop();
          myThread = null;
     }

// readChat has the toughest job in reading the message, and it's not
// really that tough. The message length is the total length of the
// bytes sent. It is followed by the length of the name of the person
// sending the chat, and then the name itself. This method has to
// compute the length of the chat string by subtracting the length of
// the name, and 4 bytes for the name length.

     public void readChat()
     throws IOException
     {
// Get the total message length
          int length = inStream.readInt();

// Get the length of the name of the person sending the chat
          int whoLength = inStream.readInt();

// Compute the length of the chat, subtract the length of the name,
// and 4 bytes for the length that was sent.

          int chatLength = length - whoLength - 4;

// Read in the name of the person sending the chat
          byte[] whoBytes = new byte[whoLength];
          inStream.readFully(whoBytes);
          String whoString = new String(whoBytes, 0);

// Read in the chat
          byte[] chatBytes = new byte[chatLength];
          inStream.readFully(chatBytes);
          String chatString = new String(chatBytes, 0);

// Pass the chat to the object that will display it

          client.incomingChat(whoString, chatString);
     }

     public void readEnter()
     throws IOException
     {
          int length = inStream.readInt();
          byte[] whoBytes = new byte[length];
          inStream.readFully(whoBytes);

          String whoString = new String(whoBytes, 0);

          client.userHasEntered(whoString);
     }

     public void readLeave()
     throws IOException
     {
          int length = inStream.readInt();
          byte[] whoBytes = new byte[length];
          inStream.readFully(whoBytes);

          String whoString = new String(whoBytes, 0);

          client.userHasLeft(whoString);
     }

     public void skipMessage()
     throws IOException
     {
          int length = inStream.readInt();
          inStream.skipBytes(length);
     }
}
